/* -*-c-*-
 *
 * $Id: acl.c,v 1.3 1999/07/27 18:30:53 mdw Exp $
 *
 * Access control list handling
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: acl.c,v $
 * Revision 1.3  1999/07/27 18:30:53  mdw
 * Various minor portability fixes.
 *
 * Revision 1.2  1999/07/26 23:28:15  mdw
 * Minor modifications for new design.
 *
 * Revision 1.1.1.1  1999/07/01 08:56:23  mdw
 * Initial revision.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <unistd.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <mLib/sub.h>

#include "acl.h"

/*----- Static variables --------------------------------------------------*/

static acl_entry *global = 0;
static acl_entry **gtail = &global;

/*----- Main code ---------------------------------------------------------*/

/* --- @acl_check@ --- *
 *
 * Arguments:	@acl_entry *a@ = pointer to ACL to check against
 *		@struct in_addr addr@ = address to check
 *
 * Returns:	Nonzero if allowed.
 *
 * Use:		Checks an address against an ACL.
 */

int acl_check(acl_entry *a, struct in_addr addr)
{
  int act = ACL_ALLOW;
  int i;

  for (i = 0; i < 2; i++) {
    for (; a; a = a->next) {
      if ((addr.s_addr & a->mask.s_addr) == a->addr.s_addr)
	return (a->act & ACL_PERM);
      act = (a->act & ACL_PERM) ^ 1;
    }
    a = global;
  }

  return (act);
}

/* --- @acl_dump@ --- *
 *
 * Arguments:	@acl_entry *a@ = pointer to ACL to dump
 *		@FILE *fp@ = pointer to stream to dump on
 *
 * Returns:	---
 *
 * Use:		Dumps an access control list to an output stream.
 */

void acl_dump(acl_entry *a, FILE *fp)
{
  if (!a)
    a = global;
  for (; a; a = a->next) {
    fprintf(fp, "  %s from ",
	    (a->act & ACL_PERM) == ACL_ALLOW ? "allow" : "deny");
    fputs(inet_ntoa(a->addr), fp);
    fputc('/', fp);
    fputs(inet_ntoa(a->mask), fp);
    fputc('\n', fp);
  }
}

/* --- @acl_free@ --- *
 *
 * Arguments:	@acl_entry *a@ = pointer to a list of ACLs
 *
 * Returns:	---
 *
 * Use:		Frees all of the memory used by an ACL.
 */

void acl_free(acl_entry *a)
{
  while (a) {
    acl_entry *aa = a;
    a = a->next;
    DESTROY(aa);
  }
}

/* --- @acl_add@ --- *
 *
 * Arguments:	@acl_entry ***a@ = address of pointer to list tail
 *		@unsigned act@ = what to do with matching addresses
 *		@struct in_addr addr, mask@ = address and mask to match
 *
 * Returns:	---
 *
 * Use:		Adds an entry to the end of an access control list.
 */

void acl_add(acl_entry ***a, unsigned act,
	     struct in_addr addr, struct in_addr mask)
{
  acl_entry *aa = CREATE(acl_entry);
  aa->act = act;
  aa->addr.s_addr = addr.s_addr & mask.s_addr;
  aa->mask = mask;
  aa->next = 0;
  if (!a)
    a = &gtail;
  **a = aa;
  *a = &aa->next;
}

/*----- That's all, folks -------------------------------------------------*/
